package pt.uc.dei.crf_ma.clustering;

import java.util.Arrays;
import java.util.HashMap;

public class DendrogramPrinter {
	private HashMap<Integer, Integer> activeLevels; 
	private int sep = 3;
	private HashMap<Integer, String> labels;
	
	public DendrogramPrinter(HashMap<Integer, String> labels) {
		this.labels = labels;
		this.activeLevels = new HashMap<Integer, Integer>(); 
	}
	
	private String hnodeToString(HNode hnode) {
		String line = "(";
		for (int i : hnode.getInstances()) {
			line += labels.get(i) + ","; 
		}
		line += ")";
		return line;
	}
	
	private boolean isPair(HNode hnode) {
		return hnode.getLeft() != null && hnode.getRight() != null;
	}
	
	private int maxHeight(HNode hnode) {
		int h = 0;
		if (isPair(hnode)) {
			int maxLeft = maxHeight(hnode.getLeft());
			int maxRight = maxHeight(hnode.getRight());
			if (maxLeft > maxRight) {
				h = maxLeft;
			} else {
				h = maxRight;
			}
		} else {
			h = hnodeToString(hnode).length();
		}
		return h + sep;
	}

	private void traverse(HNode hnode, int h, int isFirst) {
		String s = "";
		if (isPair(hnode)) {
			traverse(hnode.getLeft(), h-sep, 1);
			for (int i=0; i<h-sep; i++)
				s += ' ';
			s += '|';
		} else {
			s = hnodeToString(hnode);
			s += ' ';
		}

		while (s.length() < h)
			s += '-';
		
		if (isFirst >= 0) {
			s += '+';
			if (isFirst != 0)
				activeLevels.put(h, 1);
			else
				activeLevels.remove(h);
		}
		
		//A.sort()
		int[] A = new int[activeLevels.keySet().size()];
		int j = 0;
		for (Integer value : activeLevels.keySet()) {
			A[j] = value;
			j++;
		}
		Arrays.sort(A);
		for (Integer L : A) {
			if (s.length() < L) {
				while (s.length() < L)
					s += ' ';
				s += '|';
			}
		}

		System.out.println(s);
		
		if (isPair(hnode))
			traverse(hnode.getRight(), h-sep, 0);
	}

	public void print(HNode hnode) {
		System.out.println();
		traverse(hnode, maxHeight(hnode), -1);
		System.out.println();
	}
	
}
