package pt.uc.dei.crf_ma.clustering;

import java.util.ArrayList;
import java.util.HashMap;

public class HNode {
	private static int nextNodeId = 1;
	private int nodeId;
	private ArrayList<Integer> instances;
	private HNode left;
	private HNode right;
	private HashMap<Integer, String> labelMap;
	private double height;
	
	public HNode(HashMap<Integer, String> labelMap) {
		this.nodeId = HNode.nextNodeId++;
		this.instances = new ArrayList<Integer>();
		this.labelMap = labelMap;
		this.height = 0;
	}
	
	public HNode(Integer instance, HashMap<Integer, String> labelMap) {
		this.nodeId = HNode.nextNodeId++;
		this.instances = new ArrayList<Integer>();
		this.instances.add(instance);
		this.labelMap = labelMap;
		this.height = 0;
	}
	
	public HNode(HNode left, HNode right, HashMap<Integer, String> labelMap, double height) {
		this.nodeId = HNode.nextNodeId++;
		this.left = left;
		this.right = right;
		this.instances = determineInstances();
		this.labelMap = labelMap;
		this.height = height;
	}
	
	public ArrayList<Integer> determineInstances() {
		this.instances = new ArrayList<Integer>();
		if (left != null) {
			this.instances.addAll(left.getInstances());
		}
		if (right != null) {
			this.instances.addAll(right.getInstances());
		}
		return this.instances;
	}

	public ArrayList<Integer> getInstances() {
		if (instances == null) {
			instances = determineInstances();
		}
		return instances;
	}

	public void setInstances(ArrayList<Integer> instances) {
		this.instances = instances;
	}

	public HNode getLeft() {
		return left;
	}

	public void setLeft(HNode left) {
		this.left = left;
	}

	public HNode getRight() {
		return right;
	}

	public void setRight(HNode right) {
		this.right = right;
	}

	public int getNodeId() {
		return nodeId;
	}

	public double getHeight() {
		return height;
	}

	public void setHeight(double height) {
		this.height = height;
	}
	
	public int getSize() {
		return this.instances.size();
	}

	@Override
	public String toString() {
		String s = "HNode" + this.nodeId + "(";
		for (int instance : this.instances) {
			s += labelMap.get(instance) + "/" + instance + ",";
		}
		s += ")";
		return s;
	}
}
