package pt.uc.dei.crf_ma.evaluator;

import java.text.DecimalFormat;

public class EvaluatorResults {
	private int numCorrectTokens;
	private int totalTokens;
	private int numCorrectSegments;
	private int numPredictedSegments;
	private int numTrueSegments;
	private double accuracy;
	private double precision;
	private double recall;
	private double f1;
	private int numIterations = 0;
	private String description;
	
	public EvaluatorResults() { }
	
	public EvaluatorResults(double accuracy, double precision, double recall,
			double f1) {
		super();
		this.accuracy = accuracy;
		this.precision = precision;
		this.recall = recall;
		this.f1 = f1;
	}

	public EvaluatorResults(int numCorrectTokens, int totalTokens,
			int numCorrectSegments, int numPredictedSegments,
			int numTrueSegments, double accuracy, double precision,
			double recall, double f1) {
		super();
		this.numCorrectTokens = numCorrectTokens;
		this.totalTokens = totalTokens;
		this.numCorrectSegments = numCorrectSegments;
		this.numPredictedSegments = numPredictedSegments;
		this.numTrueSegments = numTrueSegments;
		this.accuracy = accuracy;
		this.precision = precision;
		this.recall = recall;
		this.f1 = f1;
	}
	
	@Override
	public String toString() {
		DecimalFormat f = new DecimalFormat("0.####");
		StringBuilder sb = new StringBuilder();
		sb.append(getDescription() + "\n");
//		sb.append("numiterations="
//				+ getNumIterations() + "\n");
//		sb.append("tokenaccuracy="
//				+ f.format(((double) getNumCorrectTokens()) / getTotalTokens()) + "\n");
		sb.append("segments true="
				+ getNumTrueSegments() + " pred=" + getNumPredictedSegments()
				+ " correct=" + getNumCorrectSegments() + " misses="
				+ (getNumTrueSegments() - getNumCorrectSegments()) + " alarms="
				+ (getNumPredictedSegments() - getNumCorrectSegments()) + "\n");
		sb.append("precision=" + f.format(getPrecision())
				+ " recall=" + f.format(getRecall()) + " f1=" + f.format(getF1()) 
				+ " (" + getNumIterations() + " iterations)");
		return sb.toString();
	}

	public int getNumCorrectTokens() {
		return numCorrectTokens;
	}

	public void setNumCorrectTokens(int numCorrectTokens) {
		this.numCorrectTokens = numCorrectTokens;
	}

	public int getTotalTokens() {
		return totalTokens;
	}

	public void setTotalTokens(int totalTokens) {
		this.totalTokens = totalTokens;
	}

	public int getNumCorrectSegments() {
		return numCorrectSegments;
	}

	public void setNumCorrectSegments(int numCorrectSegments) {
		this.numCorrectSegments = numCorrectSegments;
	}

	public int getNumPredictedSegments() {
		return numPredictedSegments;
	}

	public void setNumPredictedSegments(int numPredictedSegments) {
		this.numPredictedSegments = numPredictedSegments;
	}

	public int getNumTrueSegments() {
		return numTrueSegments;
	}

	public void setNumTrueSegments(int numTrueSegments) {
		this.numTrueSegments = numTrueSegments;
	}

	public double getAccuracy() {
		return accuracy;
	}

	public void setAccuracy(double accuracy) {
		this.accuracy = accuracy;
	}

	public double getPrecision() {
		return precision;
	}

	public void setPrecision(double precision) {
		this.precision = precision;
	}

	public double getRecall() {
		return recall;
	}

	public void setRecall(double recall) {
		this.recall = recall;
	}

	public double getF1() {
		return f1;
	}

	public void setF1(double f1) {
		this.f1 = f1;
	}

	public int getNumIterations() {
		return numIterations;
	}

	public void setNumIterations(int numIterations) {
		this.numIterations = numIterations;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}
	
	
}
