package pt.uc.dei.crf_ma.pipe;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.LineNumberReader;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Reader;
import java.io.Serializable;
import java.util.HashMap;
import java.util.HashSet;

import cc.mallet.pipe.Pipe;
import cc.mallet.types.Instance;
import cc.mallet.types.Token;
import cc.mallet.types.TokenSequence;

public class DirectoryLexiconMembership extends Pipe implements Serializable
{
	private String prefix;
	private String directoryPath;
	private HashMap<String, HashSet<String>> lexicons;
	boolean ignoreCase;
	
	public DirectoryLexiconMembership(String prefix, String directoryPath, boolean ignoreCase) {
		this.prefix = prefix;
		this.directoryPath = directoryPath;
		this.ignoreCase = ignoreCase;
		this.lexicons = loadLexicons(directoryPath);
	}
	
	private HashMap<String, HashSet<String>> loadLexicons(String directoryPath) {
		HashMap<String, HashSet<String>> lexicons = new HashMap<String, HashSet<String>>();
		
		File dir = new File(getClass().getResource(directoryPath).getPath());
		System.out.println(getClass().getResource(directoryPath).getPath());
		File[] lexiconFiles = dir.listFiles();
		for(File lexiconFile : lexiconFiles) {
			if(lexiconFile.getName().endsWith(".txt")) {
				try {
					BufferedReader br = new BufferedReader(new FileReader(lexiconFile));
					HashSet<String> lexicon = new HashSet<String>();
					String line = null;
					while((line = br.readLine()) != null) {
						lexicon.add(line.trim());
					}
					lexicons.put(lexiconFile.getName().replace(".txt", ""), lexicon);
				} catch (FileNotFoundException e) {
					e.printStackTrace();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
		
		return lexicons;
	}
	
	public Instance pipe (Instance carrier)
	{
		TokenSequence ts = (TokenSequence) carrier.getData();
		for (int i = 0; i < ts.size(); i++) {
			Token t = ts.get(i);
			String s = t.getText();
			
			for(String lexiconName : lexicons.keySet()) {
				HashSet<String> lexicon = lexicons.get(lexiconName);
				if(lexicon.contains(s)) {
					t.setFeatureValue(prefix + lexiconName, 1.0);
				}
			}
		}
		return carrier;
	}
	
	// Serialization 
	private static final long serialVersionUID = 1;
	private static final int CURRENT_SERIAL_VERSION = 0;
	
	private void writeObject (ObjectOutputStream out) throws IOException {
		out.writeInt(CURRENT_SERIAL_VERSION);
		out.writeObject(prefix);
		out.writeObject(directoryPath);
		out.writeBoolean(ignoreCase);
	}
	
	private void readObject (ObjectInputStream in) throws IOException, ClassNotFoundException {
		int version = in.readInt();
		this.prefix = (String) in.readObject();
		this.directoryPath = (String) in.readObject();
		this.ignoreCase = in.readBoolean();
		this.lexicons = loadLexicons(directoryPath);
	}
}